(* file: checks.ml
   author: Bob Muller

   Code for CS1103 Computer Science I Honors

   This draws a 2D grid of randomly colored squares. 
*)
open World
open Image
open Color
open Cs1103

let pi = acos(-1.)
let displayWidth = 800.0
let displayHeight = displayWidth

(* addColumnOfChecks : float -> int -> float -> Image.t -> Image.text

   The call (addColumnOfChecks x n side image)

   adds a vertical column of n randomly colored checks at the
  x-coordinate to the image.
 *)
let addColumnOfChecks n x side image =
  let empty = Image.rectangle side displayHeight red in
  let rec addChecks n image =
    match n = 0 with
    | true  -> image
    | false ->
      let y = side *. (float (n - 1)) in
      let color = Cs1103.randomColor () in
      let square = Image.rectangle side side color in
      let newImage = Image.place_image square (0.0, y) image
      in
      addChecks (n - 1) newImage in
  let column = addChecks n empty
  in
  Image.place_image column (x, 0.0) image

(* checks : unit -> Image.t
 *)
let checks () =
  let n = 5 (* 1 + (Random.int 16) *) in
  let side = displayWidth /. (float n) in
  let empty = Image.empty_scene displayWidth displayHeight in
  let rec addColumnsOfChecks m image =
    match m = 0 with
    | true  -> image
    | false ->
      let x = side *. (float (m - 1)) in
      let newImage = addColumnOfChecks n x side image
      in
      addColumnsOfChecks (m - 1) newImage
  in
  addColumnsOfChecks n empty


let () =
  World.big_bang ()
    ~name:"Checks"
    ~width:(f2I displayWidth)
    ~height:(f2I displayHeight)
    ~to_draw:checks
    ~rate:0.5
