(* file: expose.ml
   author: Bob Muller

   Code for CS1103 Computer Science I Honors

   This draws a 2D grid of colored squares. They are partially
  hidden by two transparent gray sheets. The sheets slide away,
  one on the x-axis, the other on the y-axis to expose the
  grid.
*)
open World
open Image
open Color
open Cs1103

let pi = acos(-1.)
let displayWidth = 800.0
let displayHeight = displayWidth

(* addColumnOfChecks : float -> int -> float -> Image.t -> Image.text

   The call (addColumnOfChecks x n side image)

   adds a vertical column of n randomly colored checks at the
  x-coordinate to the image.
 *)
let addColumnOfChecks n x side image =
  let empty = Image.rectangle side displayHeight red in
  let rec addChecks n image =
    match n = 0 with
    | true  -> image
    | false ->
      let y = side *. (float (n - 1)) in
      let color = Cs1103.randomColor () in
      let square = Image.rectangle side side color in
      let newImage = Image.place_image square (0.0, y) image
      in
      addChecks (n - 1) newImage in
  let column = addChecks n empty
  in
  Image.place_image column (x, 0.0) image

(* checks : unit -> Image.t
 *)
let checks () =
  let n = 5 (* 1 + (Random.int 16) *) in
  let side = displayWidth /. (float n) in
  let empty = Image.empty_scene displayWidth displayHeight in
  let rec addColumnsOfChecks m image =
    match m = 0 with
    | true  -> image
    | false ->
      let x = side *. (float (m - 1)) in
      let newImage = addColumnOfChecks n x side image
      in
      addColumnsOfChecks (m - 1) newImage
  in
  addColumnsOfChecks n empty

(* expose : Image.t -> float -> Image.t

   The call (expose picture xy) gradually withdraws two
  partially transparent screens to fully expose picture.
 *)
let expose picture xy =
  let shade = 32 in
  let color = Color.make_color shade shade shade ~alpha:160 in
  let screen = Image.rectangle displayWidth displayHeight color in
  let oneScreen = Image.place_image screen (0., xy) picture
  in
  Image.place_image screen (xy, 0.) oneScreen

(* change : float -> World.t

   This function is called on each clock tick by the World.big_bang
   function. xy is a float representing the state of the world.

   NB: The state of the world is changed by the addition of 10.0.
  The big_bang function requires us to wrap up this new state using
   the World.World constructor.
 *)
let change xy = World.World (xy +. 10.0)

(* finished : float -> bool
 *)
let finished xy = xy > displayHeight

(* drawLast : Image.t -> float -> Image.t
 *)
let drawLast picture _ = picture

let () =
  let picture = checks ()
  in
  World.big_bang 0.0
    ~name:"Expose a Picture"
    ~width:(f2I displayWidth)
    ~height:(f2I displayHeight)
    ~to_draw:(expose picture)
    ~on_tick:change
    ~rate:0.1
    ~stop_when:finished
    ~to_draw_last:(drawLast picture)
